searchNodes=[{"doc":"Ueberauth is two-phase authentication framework that provides a clear API - allowing for many strategies to be created and shared within the community. It is heavily inspired by Omniauth. You could call it a port but it is significantly different in operation - but almost the same by concept. Huge hat tip to Intridea.Ueberauth provides only the initial authentication challenge, (initial OAuth flow, collecting the information from a login form, etc). It does not authenticate each request, that's up to your application. You could issue a token or put the result into a session for your applications needs. Libraries like Guardian can help you with that aspect of authentication.The two phases are request and callback. These phases are implemented by Strategies.StrategiesStrategies are plugs that decorate or intercept requests (or both).Strategies implement the two phases and then may allow the request to flow through to your downstream plugs. Implementing the request and callback phases is optional depending on the strategies requirements. If a strategy does not redirect, the request will be decorated with Ueberauth information and allowed to carry on through the pipeline.Request PhaseThe request phase is where you request information about the user. This could be a redirect to an OAuth2 authorization url or a form for collecting username and password. The request phase is concerned with only the collection of information. When a request comes in on the request phase url the relevant strategy will receive the Ueberauth.Strategy.handle_request!/1 call.In some cases (default) the application using Ueberauth is responsible for implementing the request phase. That is, you should setup a route to receive the request phase and provide a form etc. In some cases, like OAuth, the request phase is used to redirect your user to a 3rd party site to fulfill the request.For example, an OAuth strategy for GitHub will receive the request phase url and stop the request, redirecting you to GitHub’s OAuth challenge url with some query parameters. Once you complete the GitHub OAuth flow, the user will be redirected back to the host site to the callback URL.Another example is simple email/password authentication. A request is made by the client to the request phase path and the host application displays a form. The strategy will likely not do anything with the incoming Ueberauth.Strategy.handle_request!/1 request and simply pass through to the application. Once the form is completed, the POST should go to the callback url where it is handled (passwords checked, users created / authenticated).Callback PhaseThe callback phase is where the fun happens. Once a successful request phase has been completed, the request phase provider (OAuth provider or host site etc) should call the callback url. The strategy will intercept the request via the Ueberauth.Strategy.handle_callback!/1. If successful it should prepare the connection so the Ueberauth.Auth struct can be created, or set errors to indicate a failure.See Ueberauth.Strategy for more information on constructing the Ueberauth.Auth struct.SetupIn your configuration file provide a list of the providers you intend to use.Exampleconfig :ueberauth, Ueberauth, providers: [ facebook: {Ueberauth.Strategy.Facebook, [opt1: &quot;value&quot;, opts2: &quot;value&quot;]}, github: {Ueberauth.Strategy.Github, [opt1: &quot;value&quot;, opts2: &quot;value&quot;]} ]This will define two providers for you. The general structure of the providers value is:config :ueberauth, Ueberauth, providers: [ &lt;provider name&gt;: {&lt;Strategy Module&gt;, [&lt;strategy options&gt;]} ]We use the configuration options for defining these to allow for dependency injection in different environments. The provider name will be used to construct request and response paths (by default) but will also be returned in the Ueberauth.Auth struct as the provider field.Once you've setup your providers, in your router you need to configure the plug to run. The plug should run before you application routes.In phoenix, plug this module in your controller:defmodule MyApp.AuthController do use MyApp.Web, :controller plug Ueberauth ... endIts URL matching is done via pattern matching rather than explicit runtime checks so your strategies will only fire for relevant requests.Now that you have this, your strategies will intercept relevant requests for each strategy for both request and callback phases. The default URLs are (for our Facebook &amp; GitHub example)# Request phase paths /auth/facebook /auth/github # Callback phase paths /auth/facebook/callback /auth/github/callbackIf you want to include only some of the providers with your plug you can specify a list of configured providersdefmodule MyApp.Admin.AuthController do use MyApp.Web, :controller plug Ueberauth, providers: [:identity], base_path: &quot;/admin/auth&quot; endThis will allow you to have different login points in your application selectively using some or all of the providers.Configuration of different Providers per OTP appIf you wish to use Ueberauth in multiple OTP apps, and configure each instance of Ueberauth with a different list of Providers, you will need to do some things differently. When providing configuration for Ueberauth, you should set anything that differs by OTP app under the name of your OTP app, for example:config :my_app, Ueberauth, providers: [ … ]Further, when using the Ueberauth plug, you should pass the :otp_app option, for example:defmodule MyApp.Admin.AuthController do use MyApp.Web, :controller plug Ueberauth, otp_app: :my_app, providers: [:identity], base_path: &quot;/admin/auth&quot; endThis ensures that in addition to globally configured values under :ueberauth, values set under your own namespace are used with priority.Customizing PathsThese paths can be configured on a per strategy basis by setting options on the provider.Note: These paths are absoluteExampleconfig :ueberauth, Ueberauth, base_path: &quot;/login&quot;, # default is &quot;/auth&quot; providers: [ identity: {Ueberauth.Strategies.Identity, [request_path: &quot;/login/identity&quot;, callback_path: &quot;/login/identity/callback&quot;]} ], json_library: Poison # or JasonCustomizing SchemesBy default, Ueberauth uses your Plug.Conn scheme.A custom scheme can be provided through the request header X-Forwarded-Proto.For example, in Nginx you can set it.proxy_set_header X-Forwarded-Proto $scheme;Another option is to override this via options to your strategy.providers: [ identity: {Ueberauth.Strategies.Identity, [callback_scheme: &quot;https&quot;]} ]Customizing PortsBy default, Ueberauth uses your Plug.Conn port.To override this via options to your strategy.providers: [ identity: {Ueberauth.Strategies.Identity, [callback_port: 4000]} ]Http MethodsBy default, all callback URLs are only available via the &quot;GET&quot; method. You can override this via options to your strategy.providers: [ identity: {Ueberauth.Strategies.Identity, [callback_methods: [&quot;POST&quot;]]} ]Strategy OptionsAll options that are passed into your strategy are available at runtime to modify the behaviour of the strategy.","ref":"Ueberauth.html","title":"Ueberauth","type":"module"},{"doc":"Fetch a successful auth from the Plug.Conn.This should only be called after the callback phase has run.","ref":"Ueberauth.html#auth/1","title":"Ueberauth.auth/1","type":"function"},{"doc":"Implements Plug.call/2","ref":"Ueberauth.html#call/2","title":"Ueberauth.call/2","type":"function"},{"doc":"Implements Plug.init/1","ref":"Ueberauth.html#init/1","title":"Ueberauth.init/1","type":"function"},{"doc":"Fetch the configured JSON library.A JSON library is required for Ueberauth to operate.In config.exs your implicit or explicit configuration is:config :ueberauth, Ueberauth, json_library: JasonOr:config :ueberauth, json_library: JasonIf you are using per-app configuration, you can also use:config :my_app, Ueberauth, json_library: JasonThe JSON library defaults to Jason but can be configured to Poison.In mix.exs you will need something like:def deps() do [ ... {:jason, &quot;&lt;version&gt;&quot;} # or {:poison, &quot;&lt;version&gt;&quot;} ] endThis file will serve underlying Ueberauth libraries as a hook to grab the configured JSON library.","ref":"Ueberauth.html#json_library/1","title":"Ueberauth.json_library/1","type":"function"},{"doc":"Request authentication against a provider.def callback(conn, %{&quot;provider_name&quot; =&gt; provider_name} = _params) do provider_config = case provider_name do &quot;github&quot; -&gt; { Ueberauth.Strategy.Github, [ default_scope: &quot;user&quot;, request_path: provider_auth_path(conn, :request, provider_name), callback_path: provider_auth_path(conn, :callback, provider_name), ]} end conn |&gt; Ueberauth.run_callback(provider_name, provider_config) |&gt; handle_callback(params, provider) end","ref":"Ueberauth.html#run_callback/4","title":"Ueberauth.run_callback/4","type":"function"},{"doc":"Request authentication against a provider.specified dynamically in arguments. For example, you can specify in a controller:def request(conn, %{&quot;provider_name&quot; =&gt; provider_name} = _params) do provider_config = case provider_name do &quot;github&quot; -&gt; { Ueberauth.Strategy.Github, [ default_scope: &quot;user&quot;, request_path: provider_auth_path(conn, :request, provider_name), callback_path: provider_auth_path(conn, :callback, provider_name), ]} end conn |&gt; Ueberauth.run_request(provider_name, provider_config) end","ref":"Ueberauth.html#run_request/4","title":"Ueberauth.run_request/4","type":"function"},{"doc":"","ref":"Ueberauth.html#t:method/0","title":"Ueberauth.method/0","type":"type"},{"doc":"","ref":"Ueberauth.html#t:path/0","title":"Ueberauth.path/0","type":"type"},{"doc":"","ref":"Ueberauth.html#t:route/0","title":"Ueberauth.route/0","type":"type"},{"doc":"","ref":"Ueberauth.html#t:routes/0","title":"Ueberauth.routes/0","type":"type"},{"doc":"The struct provided to indicate a success of the callback phase.This struct is constructed by the strategy by using functions defined in the strategy and is provided to the downstream plug in the connections assigns :ueberauth_auth key.","ref":"Ueberauth.Auth.html","title":"Ueberauth.Auth","type":"module"},{"doc":"","ref":"Ueberauth.Auth.html#valid?/1","title":"Ueberauth.Auth.valid?/1","type":"function"},{"doc":"","ref":"Ueberauth.Auth.html#t:t/0","title":"Ueberauth.Auth.t/0","type":"type"},{"doc":"Provides information about the credentials of a request.","ref":"Ueberauth.Auth.Credentials.html","title":"Ueberauth.Auth.Credentials","type":"module"},{"doc":"","ref":"Ueberauth.Auth.Credentials.html#t:t/0","title":"Ueberauth.Auth.Credentials.t/0","type":"type"},{"doc":"Provides a place for all raw information that was accumulated during the processing of the callback phase.","ref":"Ueberauth.Auth.Extra.html","title":"Ueberauth.Auth.Extra","type":"module"},{"doc":"","ref":"Ueberauth.Auth.Extra.html#t:t/0","title":"Ueberauth.Auth.Extra.t/0","type":"type"},{"doc":"Provides a place within the Ueberauth.Auth struct for information about the user.","ref":"Ueberauth.Auth.Info.html","title":"Ueberauth.Auth.Info","type":"module"},{"doc":"","ref":"Ueberauth.Auth.Info.html#valid?/1","title":"Ueberauth.Auth.Info.valid?/1","type":"function"},{"doc":"","ref":"Ueberauth.Auth.Info.html#t:t/0","title":"Ueberauth.Auth.Info.t/0","type":"type"},{"doc":"The struct provided to indicate a failure of authentication.All errors are provided by the relevant strategy.","ref":"Ueberauth.Failure.html","title":"Ueberauth.Failure","type":"module"},{"doc":"","ref":"Ueberauth.Failure.html#t:t/0","title":"Ueberauth.Failure.t/0","type":"type"},{"doc":"A specific error for a failed authentication attempt.The message_key may be used to identify fields or other machine interpreted methods like translation. The message field is for a human readable message indicating the cause of the error.","ref":"Ueberauth.Failure.Error.html","title":"Ueberauth.Failure.Error","type":"module"},{"doc":"","ref":"Ueberauth.Failure.Error.html#t:t/0","title":"Ueberauth.Failure.Error.t/0","type":"type"},{"doc":"The Strategy is the work-horse of the system.Strategies are implemented outside this library to meet your needs, the strategy provides a consistent API and behaviour.Each strategy operates through two phases.request phasecallback phaseThese phases can be understood with the following psuedocode.Request Phaserequest (for the request phase - default /auth/:provider) |&gt; relevant_strategy.handle_request!(conn) |&gt; continue with request plug pipelineThe request phase follows normal plug pipeline behaviour. The request will not continue if the strategy halted the connection.Callback Phaserequest (for a callback phase - default /auth/:provider/callback) |&gt; relevant_strategy.handle_auth!(conn) if connection does not have ueberauth failure |&gt; set ueberauth auth with relevant_strategy.auth |&gt; cleanup from the strategy with relevant_strategy.handle_cleanup! |&gt; continue with plug pipelineThe callback phase is essentially a decorator and does not usually redirect or halt the request. Its result is that one of two cases will end up in your connections assigns when it reaches your controller.On Failure - An Ueberauth.Failure struct is available at :ueberauth_failureOn Success - An Ueberauth.Auth struct is available at :ueberauth_authAn exampleThe simplest example is an email/password strategy. This does not intercept the request and just decorates it with the Ueberauth.Auth struct. (it is always successful)defmodule Ueberauth.Strategies.Identity do use Ueberauth.Strategy alias Ueberauth.Auth.Credentials alias Ueberauth.Auth.Extra def uid(conn), do: conn.params[&quot;email&quot;] def extra(conn), do: struct(Extra, raw_info: conn.params) def credentials(conn) do %Credentials{ other: %{ password: conn.params[&quot;password&quot;], password_confirmation: conn.params[&quot;password_confirmation&quot;] } } end endAfter the strategy has run through the handle_callback!/1 function, since there are no errors added, Ueberauth will add the constructed auth struct to the connection.The Auth struct is constructed like:def auth(conn, strategy) do %Auth{ provider: Ueberauth.Strategy.Helpers.strategy_name(conn), strategy: strategy, uid: strategy.uid(conn), info: strategy.info(conn), extra: strategy.extra(conn), credentials: strategy.credentials(conn) } endEach component of the struct is a separate function and receives the connection object. From this Ueberauth will construct and assign the struct for processing in your own controller.Redirecting during the request phaseMany strategies may require a redirect (looking at you OAuth). To do this, implement the handle_request!/1 function.def handle_request!(conn) do callback_url = callback_url(conn) redirect!(conn, callback_url) endCallback phaseThe callback phase may not do anything other than instruct the strategy where to get the information to construct the auth struct. In that case define the functions for the components of the struct and fetch the information from the connection struct.In the case where you do need to take some other step, the handle_callback!/1 function is where its at.def handle_callback!(conn) do conn |&gt; call_external_service_and_assign_result_to_private end def uid(conn) do fetch_from_my_private_area(conn, :username) end def handle_cleanup!(conn) do remove_my_private_area(conn) endThis provides a simplistic psuedocode look at what a callback + cleanup phase might look like. By setting the result of your call to the external service in the connections private assigns, you can use that to construct the auth struct in the auth component functions. Of course, as a good citizen you also cleanup the connection before the request continues.Cleanup phaseThe cleanup phase is provided for you to be a good citizen and clean up after your strategy. During the callback phase, you may need to temporarily store information in the private section of the conn struct. Once this is done, the cleanup phase exists to cleanup that temporary storage after the strategy has everything it needs.Implement the handle_cleanup!/1 function and return the cleaned conn struct.Adding errors during callbackYou have two options when you're in the callback phase. Either you can let the connection go through and Ueberauth will construct the auth hash for you, or you can add errors.You should add errors before you leave your handle_callback!/1 function.def handle_callback!(conn) do errors = [] if (something_bad), do: errors = [error(&quot;error_key&quot;, &quot;Some message&quot;) | errors] if (length(errors) &gt; 0) do set_errors!(errors) else conn end endOnce you've set errors, Ueberauth will not set the auth struct in the connections assigns at :ueberauth_auth, instead it will set a Ueberauth.Failure struct at :ueberauth_failure with the information provided detailing the failure.","ref":"Ueberauth.Strategy.html","title":"Ueberauth.Strategy","type":"behaviour"},{"doc":"When defining your own strategy you should use Ueberauth.Strategy.This provides default callbacks for all required callbacks to meet the Ueberauth.Strategy behaviour and imports some helper functions found in Ueberauth.Strategy.HelpersImportsUeberauth.Stratgey.HelpersPlug.ConnDefault OptionsWhen using the strategy you can pass a keyword list for default options:defmodule MyStrategy do use Ueberauth.Strategy, some: &quot;options&quot; # … end MyStrategy.default_options # [ some: &quot;options&quot; ]These options are made available to your strategy at YourStrategy.default_options. On a per usage level, other options can also be passed to the strategy to provide customization.Cross-Site Request ForgeryBy default strategies must implement https://tools.ietf.org/html/rfc6749#section-10.12 if you wish to disable this feature, you can use the :ignores_csrf_attack option:defmodule MyStrategy do use Ueberauth.Strategy, ignores_csrf_attack: true # … endWe strongly recommend never disabling this feature, unless you have some technical limitations that forces you to do so.To change the SameSite attribute of the cookie holding the state parameter, you can use the :state_param_cookie_same_site option:defmodule MyStrategy do use Ueberauth.Strategy, state_param_cookie_same_site: &quot;None&quot; # … end","ref":"Ueberauth.Strategy.html#__using__/1","title":"Ueberauth.Strategy.__using__/1","type":"macro"},{"doc":"Provides the credentials for the user.This is one of the component functions that is used to construct the auth struct. What you return here will be in the auth struct at the credentials key.","ref":"Ueberauth.Strategy.html#c:credentials/1","title":"Ueberauth.Strategy.credentials/1","type":"callback"},{"doc":"Returns the default options configuration of the strategy.","ref":"Ueberauth.Strategy.html#c:default_options/0","title":"Ueberauth.Strategy.default_options/0","type":"callback"},{"doc":"Provides the extra params for the user.This is one of the component functions that is used to construct the auth struct. What you return here will be in the auth struct at the extra key.You would include any additional information within extra that does not fit in either info or credentials","ref":"Ueberauth.Strategy.html#c:extra/1","title":"Ueberauth.Strategy.extra/1","type":"callback"},{"doc":"The callback phase implementation for your strategy.In this function you should make any external calls you need, check for errors etc. The result of this phase is that either a failure (Ueberauth.Failure) will be assigned to the connections assigns at ueberauth_failure or an Ueberauth.Auth struct will be constrcted and added to the assigns at :ueberauth_auth.","ref":"Ueberauth.Strategy.html#c:handle_callback!/1","title":"Ueberauth.Strategy.handle_callback!/1","type":"callback"},{"doc":"The cleanup phase implementation for your strategy.The cleanup phase runs after the callback phase and is present to provide a mechanism to cleanup any temporary data your strategy may have placed in the connection.","ref":"Ueberauth.Strategy.html#c:handle_cleanup!/1","title":"Ueberauth.Strategy.handle_cleanup!/1","type":"callback"},{"doc":"The request phase implementation for your strategy.Setup, redirect or otherwise in here. This is an information gathering phase and should provide the end user with a way to provide the information required for your application to authenticate them.","ref":"Ueberauth.Strategy.html#c:handle_request!/1","title":"Ueberauth.Strategy.handle_request!/1","type":"callback"},{"doc":"Provides the info for the user.This is one of the component functions that is used to construct the auth struct. What you return here will be in the auth struct at the info key.","ref":"Ueberauth.Strategy.html#c:info/1","title":"Ueberauth.Strategy.info/1","type":"callback"},{"doc":"Provides the uid for the user.This is one of the component functions that is used to construct the auth struct. What you return here will be in the auth struct at the uid key.","ref":"Ueberauth.Strategy.html#c:uid/1","title":"Ueberauth.Strategy.uid/1","type":"callback"},{"doc":"Provides helper methods for use within your strategy.These helpers are provided as a convenience for accessing the options passed to the specific pipelined strategy, considering the pipelined options and falling back to defaults.","ref":"Ueberauth.Strategy.Helpers.html","title":"Ueberauth.Strategy.Helpers","type":"module"},{"doc":"Add state parameter to the %Plug.Conn{}.","ref":"Ueberauth.Strategy.Helpers.html#add_state_param/2","title":"Ueberauth.Strategy.Helpers.add_state_param/2","type":"function"},{"doc":"Is the current request http method one of the allowed callback methods?","ref":"Ueberauth.Strategy.Helpers.html#allowed_callback_method?/1","title":"Ueberauth.Strategy.Helpers.allowed_callback_method?/1","type":"function"},{"doc":"The configured allowed callback http methods.This will use any supplied options from the configuration, but fallback to the default options","ref":"Ueberauth.Strategy.Helpers.html#allowed_callback_methods/1","title":"Ueberauth.Strategy.Helpers.allowed_callback_methods/1","type":"function"},{"doc":"Build params for callbackThis method will filter conn.params with whitelisted params from :callback_params settings","ref":"Ueberauth.Strategy.Helpers.html#callback_params/2","title":"Ueberauth.Strategy.Helpers.callback_params/2","type":"function"},{"doc":"The callback path for the requests strategy.When a client hits this path, the callback phase will be triggered for the strategy.","ref":"Ueberauth.Strategy.Helpers.html#callback_path/1","title":"Ueberauth.Strategy.Helpers.callback_path/1","type":"function"},{"doc":"The full URL for the callback phase for the requests strategy.The URL is based on the current requests host and scheme.","ref":"Ueberauth.Strategy.Helpers.html#callback_url/2","title":"Ueberauth.Strategy.Helpers.callback_url/2","type":"function"},{"doc":"A helper for constructing error entries on failure.The message_key is intended for use by machines for translations etc. The message is a human readable error message.Exampleerror(&quot;something_bad&quot;, &quot;Something really bad happened&quot;)","ref":"Ueberauth.Strategy.Helpers.html#error/2","title":"Ueberauth.Strategy.Helpers.error/2","type":"function"},{"doc":"The full list of options passed to the strategy in the configuration.","ref":"Ueberauth.Strategy.Helpers.html#options/1","title":"Ueberauth.Strategy.Helpers.options/1","type":"function"},{"doc":"Redirects to a url and halts the plug pipeline.","ref":"Ueberauth.Strategy.Helpers.html#redirect!/2","title":"Ueberauth.Strategy.Helpers.redirect!/2","type":"function"},{"doc":"The request path for the strategy to hit.Requests to this path will trigger the request_phase of the strategy.","ref":"Ueberauth.Strategy.Helpers.html#request_path/1","title":"Ueberauth.Strategy.Helpers.request_path/1","type":"function"},{"doc":"The full url for the request phase for the requests strategy.The URL is based on the current requests host and scheme.","ref":"Ueberauth.Strategy.Helpers.html#request_url/2","title":"Ueberauth.Strategy.Helpers.request_url/2","type":"function"},{"doc":"Sets a failure onto the connection containing a List of errors.During your callback phase, this should be called to 'fail' the authentication request and include a collection of errors outlining what the problem is.Note this changes the conn object and should be part of your returned connection of the callback_phase!.","ref":"Ueberauth.Strategy.Helpers.html#set_errors!/2","title":"Ueberauth.Strategy.Helpers.set_errors!/2","type":"function"},{"doc":"The strategy module that is being used for the request.","ref":"Ueberauth.Strategy.Helpers.html#strategy/1","title":"Ueberauth.Strategy.Helpers.strategy/1","type":"function"},{"doc":"Provides the name of the strategy or provider name.This is defined in your configuration as the provider name.","ref":"Ueberauth.Strategy.Helpers.html#strategy_name/1","title":"Ueberauth.Strategy.Helpers.strategy_name/1","type":"function"},{"doc":"Add state parameter to the options.","ref":"Ueberauth.Strategy.Helpers.html#with_state_param/2","title":"Ueberauth.Strategy.Helpers.with_state_param/2","type":"function"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Add state_param_cookie_same_site to strategy options to support different SameSite values #148","ref":"changelog.html#0-10-0-2022-06-16","title":"Changelog - 0.10.0 - 2022-06-16","type":"extras"},{"doc":"Prefer x-forwarded-host to construct callback_url #161","ref":"changelog.html#v0-9-0-2022-04-27","title":"Changelog - v0.9.0 - 2022-04-27","type":"extras"},{"doc":"Add support for custom URL schemes #144","ref":"changelog.html#v0-8-0-2021-08-19","title":"Changelog - v0.8.0 - 2021-08-19","type":"extras"},{"doc":"Add support for CSRF #136Improve documentation #137","ref":"changelog.html#v0-7-0-2020-04-17","title":"Changelog - v0.7.0 - 2020-04-17","type":"extras"},{"doc":"Dynamic providersBirthday part of info struct","ref":"changelog.html#v0-6-3-2020-03-05","title":"Changelog - v0.6.3 - 2020-03-05","type":"extras"},{"doc":"Fixed Ueberauth request not respecting Script Name #97","ref":"changelog.html#v0-6-2-2019-09-11","title":"Changelog - v0.6.2 - 2019-09-11","type":"extras"},{"doc":"Fix versioning for plug dependency","ref":"changelog.html#v0-6-1-2019-03-14","title":"Changelog - v0.6.1 - 2019-03-14","type":"extras"},{"doc":"Target Elixir 1.3 and aboveFix Elixir 1.4 warningsFix bug preventing multiple providers","ref":"changelog.html#v0-4-0-2016-09-21","title":"Changelog - v0.4.0 - 2016-09-21","type":"extras"},{"doc":"Allow :redirect_url to be configuredHandle requests with or without trailing slash","ref":"changelog.html#v0-3-0-2016-07-19","title":"Changelog - v0.3.0 - 2016-07-19","type":"extras"},{"doc":"Add the ability to select which providers to use on a per-plug basis","ref":"changelog.html#v0-2-1-2015-12-23","title":"Changelog - v0.2.1 - 2015-12-23","type":"extras"},{"doc":"Remove the Ueberauth.plug function in favour of making Ueberauth a plug","ref":"changelog.html#v0-2-0-2015-11-28","title":"Changelog - v0.2.0 - 2015-11-28","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v0-1-0-2015-11-15","title":"Changelog - v0.1.0 - 2015-11-15","type":"extras"},{"doc":"Überauth An Elixir Authentication System for Plug-based Web ApplicationsUeberauth is two-phase authentication framework that provides a clear API - allowing for many strategies to be created and shared within the community. It is heavily inspired by Omniauth. You could call it a port but it is significantly different in operation - but almost the same concept. Huge hat tip to Intridea.Ueberauth provides only the initial authentication challenge, (initial OAuth flow, collecting the information from a login form, etc). It does not authenticate each request, that's up to your application. You could issue a token or put the result into a session for your applications needs. Libraries like Guardian can help you with that aspect of authentication.The two phases are request and callback. These phases are implemented by Strategies.","ref":"readme.html","title":"Überauth","type":"extras"},{"doc":"Strategies are plugs that decorate or intercept requests (or both).Strategies implement the two phases and then may allow the request to flow through to your downstream plugs. Implementing the request and callback phases is optional depending on the strategies requirements. If a strategy does not redirect, the request will be decorated with Ueberauth information and allowed to carry on through the pipeline.See the full list of the strategies on the Wiki.","ref":"readme.html#strategies","title":"Überauth - Strategies","type":"extras"},{"doc":"The request phase is where you request information about the user. This could be a redirect to an OAuth2 authorization url or a form for collecting username and password. The request phase is concerned with only the collection of information. When a request comes in on the request phase url the relevant strategy will receive the handle_request! call.In some cases (default) the application using Ueberauth is responsible for implementing the request phase. That is, you should set up a route to receive the request phase and provide a form etc. In some cases, like OAuth, the request phase is used to redirect your user to a 3rd party site to fulfill the request.For example, an OAuth strategy for GitHub will receive the request phase url and stop the request, redirecting you to GitHub’s OAuth challenge url with some query parameters. Once you complete the GitHub OAuth flow, the user will be redirected back to the host site to the callback URL.Another example is simple email/password authentication. A request is made by the client to the request phase path and the host application displays a form. The strategy will likely not do anything with the incoming handle_request! request and simply pass through to the application. Once the form is completed, the POST should go to the callback url where it is handled (passwords checked, users created / authenticated).","ref":"readme.html#request-phase","title":"Überauth - Request Phase","type":"extras"},{"doc":"The callback phase is where the fun happens. Once a successful request phase has been completed, the request phase provider (OAuth provider or host site, etc) should call the callback URL. The strategy will intercept the request via the handle_callback!. If successful, it should prepare the connection so the Ueberauth.Auth struct can be created, or set errors to indicate a failure.See Ueberauth.Strategy for more information on constructing the Ueberauth.Auth struct.Looking for an example? Take a look ueberauth/ueberauth_example.","ref":"readme.html#callback-phase","title":"Überauth - Callback Phase","type":"extras"},{"doc":"Add the dependency# mix.exs defp deps do # Add the dependency [{:ueberauth, &quot;~&gt; 0.7&quot;}] endFetch the dependenciesmix deps.get","ref":"readme.html#setup","title":"Überauth - Setup","type":"extras"},{"doc":"In your configuration file (config/config.exs) provide a list of the providers you intend to use. For example:config :ueberauth, Ueberauth, providers: [ facebook: { Ueberauth.Strategy.Facebook, [ opt1: &quot;value&quot;, opts2: &quot;value&quot; ] }, github: { Ueberauth.Strategy.Github, [ opt1: &quot;value&quot;, opts2: &quot;value&quot; ] } ]This will define two providers for you. The general structure of the providers value is:config :ueberauth, Ueberauth, providers: [ &lt;provider name&gt;: { &lt;Strategy Module&gt;, [ &lt;strategy options&gt; ] } ]We use the configuration options for defining these to allow for dependency injection in different environments. The provider name will be used to construct request and response paths (by default) but will also be returned in the Ueberauth.Auth struct as the provider field.Once you've setup your providers, in your router you need to configure the plug to run. The plug should run before your application routes.In phoenix, plug this module in your controller:defmodule MyApp.AuthController do use MyApp.Web, :controller plug Ueberauth ... endIts URL matching is done via pattern matching rather than explicit runtime checks so your strategies will only fire for relevant requests.Now that you have this, your strategies will intercept relevant requests for each strategy for both request and callback phases. The default urls are (for our Facebook &amp; GitHub example)# Request phase paths /auth/facebook /auth/github # Callback phase paths /auth/facebook/callback /auth/github/callback","ref":"readme.html#configuring-providers","title":"Überauth - Configuring providers","type":"extras"},{"doc":"These paths can be configured on a per strategy basis by setting options on the provider.Note: These paths are absoluteconfig :ueberauth, Ueberauth, base_path: &quot;/login&quot;, # default is &quot;/auth&quot; providers: [ identity: {Ueberauth.Strategies.Identity, [request_path: &quot;/login/identity&quot;, callback_path: &quot;/login/identity/callback&quot;]} ]","ref":"readme.html#customizing-paths","title":"Überauth - Customizing Paths","type":"extras"},{"doc":"Your JSON serializer can be configured depending on what you have installed in your application. Defaults to Jason.config :ueberauth, Ueberauth, json_library: Poison # default is Jason","ref":"readme.html#customizing-json-serializer","title":"Überauth - Customizing JSON Serializer","type":"extras"},{"doc":"By default, all callback URLs are only available via the &quot;GET&quot; method. You can override this via options to your strategy.providers: [ identity: {Ueberauth.Strategies.Identity, [callback_methods: [&quot;POST&quot;]]} ]","ref":"readme.html#http-methods","title":"Überauth - HTTP Methods","type":"extras"},{"doc":"All options that are passed into your strategy are available at runtime to modify the behaviour of the strategy.","ref":"readme.html#strategy-options","title":"Überauth - Strategy Options","type":"extras"},{"doc":"Copyright (c) 2015 Sonny ScrogginReleased under the MIT License, which can be found in the repository in LICENSE.","ref":"readme.html#copyright-and-license","title":"Überauth - Copyright and License","type":"extras"},{"doc":"Contributing to Ueberauth","ref":"contributing.html","title":"Contributing to Ueberauth","type":"extras"},{"doc":"fork ueberauthcreate a topic branchmake logically-grouped commits with clear commit messagespush commits to your forkopen a pull request against uerberauth/masterbump the version in the mix.exs following SemVer.add an entry to CHANGELOG.md","ref":"contributing.html#pull-requests-welcome","title":"Contributing to Ueberauth - Pull Requests Welcome","type":"extras"},{"doc":"If you believe there to be a bug, please provide the maintainers with enough detail to reproduce or a link to an app exhibiting unexpected behavior. For help, please start with Stack Overflow.","ref":"contributing.html#issues","title":"Contributing to Ueberauth - Issues","type":"extras"}]